;
;        _--__------___
;            /   /
;PA0 DOT PA1 DASH PA2 KEY PA3 TONE PA4 PWM PA5 UP PA6 DOWN PA7 STKEY
;PB0 TUP PB1 TDN
RTIFR   equ     $2
TOFR    equ     $3
RTIF    equ     $6
IRQR    equ     $1
IRQF    equ     $3
TOF     equ     $7
PORTA   equ     $0
PORTB   equ     $1
DDRA    equ     $4
DDRB    equ     $5
TSCR    equ     $8
ISCR    equ     $A

IDLEV   equ     $0
DOTV    equ     $1
DASHV   equ     $2
SMSPCV  equ     $3
LTSPCV  equ     $4
KEYSTA  equ     $0              ;Output key status
NODOT   equ     $1              ;Don't test DOT paddle
NODASH  equ     $2              ;Don't test DASH paddle
DOTC    equ     $3              ;DOT paddle closed
DASHC   equ     $4              ;DASH paddle closed
PREDOT  equ     $5              ;Previous is a DOT

DOTP    equ     $1              ;DOT paddle  PORTB
DASHP   equ     $0              ;DASH paddle PORTB
SP_UP   equ     $0              ;Speed up key
SP_DN   equ     $1              ;Speed down key
AUTOSP  equ     $2              ;Auto space
KEYIN   equ     $3              ;Key input
CODE    equ     $4              ;Morse code output
PWMOUT  equ     $5              ;PWM speed output
TONE    equ     $6              ;Keyer sidetone output about 1Khz
KEY     equ     $7              ;Keyer output

BASE_H  equ     $09             ;BASE/WPM=WPM count
BASE_L  equ     $28             ;4Mhz crystal 976.6Hz
TOPWPM  equ     !96             ;Top value of WPM
BTMWPM  equ     !6              ;Bottom value of WPM
INIWPM  equ     !30             ;WPM initial value

*
*  Mask Option Register
*
       ORG      $17
       db       %10000001       ;MOR

*
*  RAM - variables
*
        org     $E0
STATE   ds      1               ;State of keyer
                                ;0 - Idle
                                ;1 - Sending DOT
                                ;2 - Sending DASH
                                ;3 - Sending space within one character
                                ;4 - Sending space between characters
CONTROL ds      1               ;Keyer control for bit operation
CLK_CNT ds      1               ;Basic clk, 1 count = DOT /2
T2W_H   ds      1
T2W_L   ds      1               ;1Khz tone to WPM counter
WPMBH   ds      1               ;WPM backup
WPMBL   ds      1
WPM_H   ds      1
WPM_L   ds      1               ;1Khz tone to WPM org
WPM     ds      1               ;WPM real value
TM_CNT  ds      1               ;TOF count
RT_CNT  ds      1               ;Real timer count
RT_CNH  ds      1               ;Real timer count high
BIN_V   ds      1               ;
LP_CNT  ds      1               ;
PWM     ds      1

        org     $200
START:  rsp
        clra
        sta     ISCR            ;NO EXT IRQ
        sta     CONTROL
        sta     STATE
        sta     DDRB
        lda     #$F0
        sta     DDRA            ;PA0-3 input PA4-7 output
        lda     #%00110000      ;TOF RTI int.
        sta     TSCR
        lda     WPM
        cmp     #TOPWPM
        bhi     RSTWPM
        cmp     #BTMWPM
        bhs     RITWPM
RSTWPM: lda     #INIWPM
RITWPM: sta     WPM
        jsr     CAL
        cli
LOOP:   brclr   SP_UP,PORTA,UPSP
        brclr   SP_DN,PORTA,DNSP
        clrx
        stx     $03F0           ;WATCH DOG FEED
        wait
        bra     LOOP

DELAY0: lda     #!60            ;Delay about 0.5s
DLYSAM: clr     RT_CNT
DELAY1: clrx
        stx     $03F0           ;WATCH DOG FEED
        cmp     RT_CNT
        bhi     DELAY1
        rts

DELAY2: lda     #!12            ;Delay about 0.1s
        bra     DLYSAM

UPSP:   bsr     SPUP
        bsr     DELAY0
UPSP2:  brset   SP_UP,PORTA,LOOP
        bsr     SPUP
        bsr     DELAY2
        bra     UPSP2

DNSP:   bsr     SPDN
        bsr     DELAY0
DNSP2:  brset   SP_DN,PORTA,LOOP
        bsr     SPDN
        bsr     DELAY2
        bra     DNSP2

SPUP:   lda     WPM
        cmp     #TOPWPM
        blo     SPUP1
        lda     #TOPWPM
SAME2:  sta     WPM
        jsr     CAL
        bra     SPUPE
SPUP1:  inc     WPM
        jsr     CAL
SPUPE:  rts

SPDN:   lda     WPM
        cmp     #BTMWPM
        bhi     SPDN1
        lda     #BTMWPM
        bra     SAME2
SPDN1:  dec     WPM
        jsr     CAL
SPDNE:  rts

TM_INT: brset   KEYSTA,CONTROL,DATDN
        bset    CODE,PORTA
        bra     WEIGHT
DATDN:  bclr    CODE,PORTA
WEIGHT: brclr   KEYIN,PORTA,KEYDN
        bclr    KEY,PORTA
        bclr    TONE,PORTA
        bra     TSTPD
KEYDN:  bset    KEY,PORTA
        bih     HIGH
        brset   $1,TM_CNT,TSTPD
HIGH:   brclr   TONE,PORTA,ROLOVR
        bclr    TONE,PORTA
        bra     TSTPD
ROLOVR: bset    TONE,PORTA      ;Down key and generate tone

TSTPD:  jsr     TSTPDL
        lda     TM_CNT
        and     #$0F
        inca
        cmp     PWM
        bhi     CLRPWM
        bset    PWMOUT,PORTA
        bra     COUNT
CLRPWM: bclr    PWMOUT,PORTA

COUNT:  lda     STATE
        beq     STAJMP
        lda     T2W_L
        beq     TSTHI
        dec     T2W_L
        bne     T2WEND
TSTHI:  lda     T2W_H
        beq     LD_NEW
        dec     T2W_L
        dec     T2W_H
T2WEND: jmp     END

LD_CNT: lda     WPM_L
        sta     T2W_L
        lda     WPM_H
        sta     T2W_H
        rts

LD_NEW: bsr     LD_CNT
STAJMP: lda     STATE
        and     #$07            ;Make sure STATE value is not bigger than
        tax                     ;$7, for the reliability
        lslx
        jmp     JMPTAB,x
JMPTAB: bra     IDLE
        bra     DOT_S
        bra     DASH_S
        bra     SM_SPC
        bra     LT_SPC
        bra     IDLE
        bra     IDLE
        bra     IDLE

DOT_S:  dec     CLK_CNT
        bne     DOTE
        lda     #$02
        sta     CLK_CNT
        lda     #SMSPCV
        sta     STATE
        bclr    KEYSTA,CONTROL
        bset    PREDOT,CONTROL
DOTE:   bra     END

DASH_S: dec     CLK_CNT
        brset   $2,CLK_CNT,DASH_2
        bclr    NODOT,CONTROL
DASH_2: bne     DASHE
        lda     #$02
        sta     CLK_CNT
        lda     #SMSPCV
        sta     STATE
        bclr    KEYSTA,CONTROL
        bclr    PREDOT,CONTROL
DASHE:  bra     END

SM_SPC: dec     CLK_CNT
        bne     SMSPCE
        lda     #%11111001
        and     CONTROL
        sta     CONTROL         ;Allow testing DOT or DASH paddle
        jsr     TSTPDL
        brclr   PREDOT,CONTROL,DASPC
        brset   DASHC,CONTROL,LDDASH
        brset   DOTC,CONTROL,LDDOT
        bra     SAME
DASPC:  brset   DOTC,CONTROL,LDDOT
        brset   DASHC,CONTROL,LDDASH
SAME:   brset   AUTOSP,PORTA,SAME1
        lda     #IDLEV
        sta     STATE
SMSPCE: bra     END
SAME1:  lda     #$04
        sta     CLK_CNT
        lda     #LTSPCV
        sta     STATE
        bra     SMSPCE

LT_SPC: dec     CLK_CNT
        bne     LTSPCE
        brset   DOTC,CONTROL,LDDOT
        brset   DASHC,CONTROL,LDDASH
        lda     #IDLEV
        sta     STATE
LTSPCE: bra     END


IDLE:   brset   DOTC,CONTROL,LDCNT1
        brset   DASHC,CONTROL,LDCNT2
        lda     #%11111001
        and     CONTROL
        sta     CONTROL         ;Allow testing DOT or DASH paddle
        clra
        sta     STATE

END:    brclr   RTIF,TSCR,END2
        inc     RT_CNT
        bset    RTIFR,TSCR
END2:   bset    TOFR,TSCR
        inc     TM_CNT
        rti


LDCNT1: jsr     LD_CNT
LDDOT:  lda     #$02
        sta     CLK_CNT
        lda     #DOTV
        sta     STATE
        lda     CONTROL
        and     #%11100111
        ora     #%00000111
        sta     CONTROL
        bra     END

LDCNT2: jsr     LD_CNT
LDDASH: lda     #$06
        sta     CLK_CNT
        lda     #DASHV
        sta     STATE
        lda     CONTROL
        and     #%11100111
        ora     #%00000111
        sta     CONTROL
        bra     END

CAL:    lda     #BASE_H
        sta     WPMBH
        lda     #BASE_L
        sta     WPMBL
        clr     BIN_V
        lda     #!16
        sta     LP_CNT
CAL1:   lsl     WPMBL
        rol     WPMBH
        rol     BIN_V
        lda     BIN_V
        sub     WPM
        blo     CAL2
        sta     BIN_V
        inc     WPMBL
CAL2:   dec     LP_CNT
        bne     CAL1
        lda     WPMBH
        sta     WPM_H
        lda     WPMBL
        sta     WPM_L
        lda     WPM
        lsra
        sub     #$03
        lsra
        cmp     #$0F
        bls     GOTPWM
        lda     #$0F
GOTPWM  sta     PWM
        rts

TSTPDL: brset   DOTP,PORTB,TSTDAH
DOTSET: brset   NODOT,CONTROL,TSTDAH
        bset    DOTC,CONTROL
TSTDAH: brset   DASHP,PORTB,PDLEND
        brset   NODASH,CONTROL,PDLEND
        bset    DASHC,CONTROL
PDLEND: rts

EXTINT: bcle    IRQE,ISCR
        rti

        org     $3F8
        dw      TM_INT                  ;TINER INT
        dw      EXTINT                  ;EXT IN
        dw      START                   ;SOFT IN
        dw      START                   ;RESET
